%% ADD TENSORLAB TO PATH
path(path,['C:\GitLab\Tools\TensorLab20\TensorLab20']);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% note units are mm and MPa
% PARAMETERS
% geometric:
H = 20; % height, in mm
L = 20; % length, in mm
thick = 200;  % thickness (in out-of-plane direction), in mm
% prescribed displacement, in mm
U = 15;
% material properties, in MPa
G     = 1000;
kappa = 3000;

% define the basis
ee = cartesianbasis2d('e1', 'e2');
e1 = ee(1);
e2 = ee(2);

% determine material coefficient tensor
I   = identity(2, ee);
I4  = identity(4, ee);
I4S = 1/2 * (I4 + rtranspose(I4));
C   = kappa * I*I + 2*G * (I4S - 1/3*I*I);

% make FE mesh
m1 = 3; % number of elements in horizontal direction
m2 = 3; % number of elements in vertical direction
[x, conn] = femgrid([ L  H ], [ m1  m2 ], ee, 'ElementType', 'Linear');
m = size(conn, 1); % number of elements in the mesh
n = size(x, 1);    % number of nodes in the mesh 

figure(2)
femplot(x, conn, 'Color' , 'black', 'Nodes', 'on', ... 
        'NodeNumbers' , 'on', 'ElementNumbers', 'off');


% coordinates of integration points 
% (in the coordinate system of the master element)
% and the weight factors
xi = [ -1/sqrt(3)*e1 - 1/sqrt(3)*e2
        1/sqrt(3)*e1 - 1/sqrt(3)*e2
        1/sqrt(3)*e1 + 1/sqrt(3)*e2
       -1/sqrt(3)*e1 + 1/sqrt(3)*e2 ];
w  = [ 1
       1
       1
       1 ];

% assemble system matrices in element loop
K = zeros(2, ee, n, n); % zero matrix of nxn 2nd order tensors in basis ee
   
   for e = 1:m % loop over all elements

    iie = conn(e, :); % nodes of the current element e
    xe  = x(iie);     % coordinates of these nodes
        
    % compute element matrices in integration point loop
    Ke = zeros(2, ee, 4, 4); % zero matrix of 4x4 2nd order tensors in basis ee
    
    for k = 1:4 % loop over integration points 
        
        xi1 = dot(xi(k), e1); % x1 coordinate of the current integr. point
        xi2 = dot(xi(k), e2); % x2 coordinate of the current integr. point

       % column of the shape functions
        Ne = [ 1/4*(1-xi1)*(1-xi2)
               1/4*(1+xi1)*(1-xi2)
               1/4*(1+xi1)*(1+xi2)
               1/4*(1-xi1)*(1+xi2) ];
        
        % column of the gradient of the shape functions
        % with respect to the local coordinates of the mater element
        gradxiNe = [ -1/4*(1-xi2)*e1 - 1/4*(1-xi1)*e2
                      1/4*(1-xi2)*e1 - 1/4*(1+xi1)*e2
                      1/4*(1+xi2)*e1 + 1/4*(1+xi1)*e2
                     -1/4*(1+xi2)*e1 + 1/4*(1-xi1)*e2 ];
        % Jacobian
        J = gradxiNe' * xe;
       
        % column of the gradient of the shape functions
        % with respect to the global coordinates of the mesh
        gradNe = dot(inv(J), gradxiNe);
        % element matrix and right hand side
        Ke = Ke + w(k) * dot(gradNe, C, gradNe') * thick * det(J);     
   
    end % end of interation point loop
   
% assembly
    K(iie, iie) = K(iie, iie) + Ke;
    
end % end of element loop

% apply boundary conditions by partitioning:
bottom = [ 1:(m1+1) ]; % nodes on the bottom boundary of the mesh
left   = [ (m1+2):(m1+1):(m1+1)*(m2) ]; % nodes on the left boundary of the mesh
right  = [ 2*(m1+1):(m1+1):(m1+1)*(m2) ]; % nodes on the right boundary of the mesh
top    = [ ((m1+1)*m2+1):(m1+1)*(m2+1) ]; % nodes on the top boundary of the mesh
% nodes where essential b.c. are applied
iiu = [bottom left right top];
% the remaining nodes
iif = setdiff(1:n, iiu);

Kuu = K(iiu, iiu);
Kfu = K(iif, iiu);
Kuf = K(iiu, iif);
Kff = K(iif, iif);

% values of the prescribed displacements
% uu  = [ zeros(1, ee, length(bottom),1) ; -U/H*dot(x(left),e2)*e2; ...
%         -U/H*dot( x(right),e2)*e2; -U*e2*ones(length(top), 1)  ]; 
uu  = [ zeros(1, ee, length(bottom),1) ; -U/H*dot(x(left),e2)*e2; ...
        -U/H*dot( x(right),e2)*e2; -U/H*dot( x(top),e2)*e2; ]; 

ff  = zeros(1, ee, length(iif), 1);

% solve the system for unknowns
uf = Kff \ (ff - dot(Kfu, uu)); 
u = zeros(1, ee, n, 1);
u(iiu) = uu;
u(iif) = uf;

fu = dot(Kuu, uu) + dot(Kuf, uf);
f = zeros(1, ee, n, 1);
f(iiu) = fu;
f(iif) = ff;

figure(1)
clf
% plot the displacement solution
quiver(x, u, 0);
axis equal
hold on
%femplot(x, conn, 'Color', 'k')
femplot(x+u, conn, 'Color', 'r');
hold off

% compute the reaction force on the punch
F = sum(f(top));
Fv = dot(F,e2)
% axial nominal stress
sigmanom = Fv/(L*thick)
 
% compute the strain and stress at right bottom corner
e   = 2; % element number
% local coordinates of the point of interest
xi1 =  1; 
xi2 = -1;

iie = conn(e, :);
xe  = x(iie);
ue  = u(iie);

gradxiNe = [ -1/4*(1-xi2)*e1 - 1/4*(1-xi1)*e2
              1/4*(1-xi2)*e1 - 1/4*(1+xi1)*e2
              1/4*(1+xi2)*e1 + 1/4*(1+xi1)*e2
             -1/4*(1+xi2)*e1 + 1/4*(1-xi1)*e2 ];

J = gradxiNe' * xe;
gradNe = dot(inv(J), gradxiNe);

gradu   = gradNe' * ue;
epsilon = 1/2 * (gradu + gradu')

sigma = ddot(C, epsilon)
