function S = tm2sm(t,b,o)
%--------------------------------------------------------------------------
% Renan Liupekevicius TU/e
% TM2SM  Transform a square tensor matrix to an equivalent scalar matrix
%   S = TM2SM(t,b,o)
%   
%   input  t - tensor matrix
%   input  b - basis
%   input  o - order of the tensor matrix component
%   output S - equivalent square scalar matrix S 
%
%--------------------------------------------------------------------------

% Comments (solved)
%--------------------------------------------------------------------------
% I had a problem for passing a object tensor to the function. For some 
% reason when inside the function I can't evaluate M(2).
% This function coded within @tensor folder will be somthing like this:
% (not correct)
%
% % cast tensor-matrix into multidimensional array
% b = t.basis;
% sm = t.size;
% st = size(b, 1) * ones(1, t.order);
% nm = length(sm);
% nt = length(st);
% c = reshape(t.components, [sm st]);
% 
% % reorder dimensions and reshape to matrix
% c = permute(c, [1:nm/2 nm+(1:nt/2) nm/2+(1:nm/2) nm+nt/2+((1:nt/2))]);
% n = prod([sm(1:nm/2) st(1:nt/2)]);
% c = reshape(c, [n n]);
%--------------------------------------------------------------------------

% spatial dimension
dim  = size(b, 1);

% component order
order = o;

if order==0
S=t;
elseif order ==1 % TENSOR MATRIX WITH 1ST-ORDER COMPONENTS
    % case 2D
    if dim==2
       
        % extract basis component
        e1 = b(1);
        e2 = b(2);
    
        % size of square tensor matrix nxn
        n   = size(t,1);
        % size of square tensor matrix nxn
        m   = size(t,2);
    
    
        % size of square scalar matrix NxM
        N   = dim * size(t,1);
        M   =  1  * size(t,2);
        
        % define scalar matrix S equivalent to t
        S = zeros(N,M);
        
        for i=1:n
           for j=1:m
           %scalar submatrix related to tensor M(i,j)    
           aux  = [dot(t(i,j),e1);
                   dot(t(i,j),e2) ];  
           % indices in the scalar matrix t
           si = 1+dim*(i-1):dim*i;
           sj = j                ;
           S(si,sj) = S(si,sj) + aux;
           end
        end
    
    % case 3D
    elseif dim==3
        
        % extract basis components
        e1 = ee(1);
        e2 = ee(2);
        e3 = ee(3);
    
        % size of square tensor matrix nxn
        n   = size(t,1);
        % size of square tensor matrix nxn
        m   = size(t,2);
    
    
        % size of square scalar matrix NxM
        N   = dim * size(t,1);
        M   =  1  * size(t,2);
        
        % define scalar matrix S equivalent to t
        S = zeros(N,M);
        
        for i=1:n
           for j=1:m
           %scalar submatrix related to tensor M(i,j)    
           aux  = [dot(t(i,j),e1) ;
                   dot(t(i,j),e1) ;
                   dot(t(i,j),e1)  ];
           % indices in the scalar matrix t
           si = 1+dim*(i-1):dim*i;
           sj = j                ;
           S(si,sj) = S(si,sj) + aux;
           end
        end
    end

elseif order == 2   % TENSOR MATRIX WITH 2ST-ORDER COMPONENTS
    % case 2D
    if dim==2
       
        % extract basis component
        e1 = b(1);
        e2 = b(2);
    
        % size of square tensor matrix nxn
        n   = size(t,1);
        % size of square tensor matrix nxn
        m   = size(t,2);
    
    
        % size of square scalar matrix NxM
        N   = dim * size(t,1);
        M   = dim * size(t,2);
        
        % define scalar matrix S equivalent to t
        S = zeros(N,M);
        
        for i=1:n
           for j=1:m
           %scalar submatrix related to tensor M(i,j)    
           aux  = [dot(e1,t(i,j),e1) dot(e1,t(i,j),e2);
                   dot(e2,t(i,j),e1) dot(e2,t(i,j),e2)];  
           % indices in the scalar matrix t
           si=1+dim*(i-1):dim*i;
           sj=1+dim*(j-1):dim*j;
           S(si,sj) = S(si,sj) + aux;
           end
        end
    
    % case 3D
    elseif dim==3
        
        % extract basis components
        e1 = ee(1);
        e2 = ee(2);
        e3 = ee(3);
    
        % size of square tensor matrix nxn
        n   = size(t,1);
        % size of square tensor matrix nxn
        m   = size(t,2);
    
    
        % size of square scalar matrix NxM
        N   = dim * size(t,1);
        M   = dim * size(t,2);
        
        % define scalar matrix S equivalent to t
        S = zeros(N,M);
        
        for i=1:n
           for j=1:m
           %scalar submatrix related to tensor M(i,j)    
           aux  = [dot(e1,t(i,j),e1) dot(e1,t(i,j),e2) dot(e1,t(i,j),e3);
                   dot(e2,t(i,j),e1) dot(e2,t(i,j),e2) dot(e2,t(i,j),e3);
                   dot(e3,t(i,j),e1) dot(e3,t(i,j),e2) dot(e3,t(i,j),e3)];
           % indices in the scalar matrix t
           si=1+dim*(i-1):dim*i;
           sj=1+dim*(j-1):dim*j;
           S(si,sj) = S(si,sj) + aux;
           end
        end
    end

else
    error("the tensor matrix component order is too high. tm2sm " + ...
          "function only transforms up to components of order 2");
end

% return a sparse matrix
S=sparse(S);

end