%% 2D spring element validation
% The goal of this script is to solve a  static problem described by:
% give the external forces applied to the certain nodes, which are the
% displacements corresponding to these forces?
%
%
%  f is the tensor vector given
%  K is the tensor stiffness matrix
%  u is the unkown
% 
%  SOlVE  K u = f
clear all; close all;
%% ADD TENSORLAB 
path(path,['C:\GitLab\Tools\TensorLab20\TensorLab20']);
path(path,["C:\GitLab\Tools\TensorLab20\TensorLab20"+...
            "\dev\shared_functions"]);
%% define the basis
b  = {'e1'; 'e2'};
ee = cartesianbasis2d(b{1}, b{2});
e1 = ee(1);
e2 = ee(2);
I   = identity(2, ee);

%% MATERIAL

k = 10; % [N/m]

%% MESH

% geometric:
H = 5; % height, in m
L = 5; % length, in m

% make FE mesh
m1 = 1; % number of elements in horizontal direction
m2 = 1; % number of elements in vertical direction
[x, conn] = femgrid([ L  H ], [ m1  m2 ], ee, 'ElementType', 'Linear');
m = size(conn, 1); % number of elements in the mesh
n = size(x, 1);    % number of nodes in the mesh 



figure
clf
femplot(x,conn,'Color' , 'black', 'Nodes', 'off', ...
'NodeNumbers' , 'on', 'ElementNumbers', 'on');
title('linear element mesh')


%% SPRING CONNECTION
% We must define a connextivity matrix that explicitly tells which nodes
% are connected with a spring of constant k.
%
% Connecting node i and node j
%
%         string element (physical)              
%
%                     k
%      i  ._________/\/\/\/\__________. j  
%
%      conns = [i j]
%

conns = [1 2
         2 4
         4 3
         3 1];


%% ASSEMBLE SYSTEM MATRIX ON A LOOP


K = zeros(2, ee, n, n); % zero matrix of nxn 2nd order tensors in basis ee

   for e = 1:size(conns,1) % loop over all string elements
    
    % extract node position from current string element   
    iie  = conns(e, :) % nodes of the current element e
    xe   = x(iie)      % coordinates of these nodes
    
    % unit vector in i-j direction in the undeformed configuration
    l0ij = (xe(2)-xe(1))/norm(xe(2)-xe(1)) 

    % compute elementary matrices                    
    Ke = k*l0ij*l0ij* [   -1     1 
                           1    -1 ];

    % assembly
    K(iie, iie) = K(iie, iie) + Ke;

   end % end of element loop


%% BOUNDARY CONDITIONS


% prescribe nodes 1 and 2 and 3 to zero
u_p = zeros(1, ee, 3, 1); %

% free nodes are 3 and 4

% external forces on (free) nodes:  4
f_f = [1*e2];   % [N]

K_f_f = K(4,4);

u_f = K_f_f\f_f;

%final solution
u  = zeros(1, ee, n, 1); %

u([1 2 3]) = u_p;
u(4)       = u_f;

%% SOLUTION
% u         =  zeros(1, ee, n, 1); %
% u([3 4])  =  u_f;
% 

figure(4)
clf
hold on
femplot(x, conn, 'Color', 'r');
title("Stationary problem (check if normalized)" );
femplot(x+u, conn, 'Color', 'k');
title("Stationary problem (check if normalized)" );
hold off