%% 2D spring element validation
% The goal of this script is to solve a  static problem described by:
% give the external forces applied to the certain nodes, which are the
% displacements corresponding to these forces?
%
%
%  f is the external force tensor vector given
%  K is the tensor stiffness matrix
%  u is the unkown
% 
%  SOlVE  K u = f
clear all; close all;

%% ADD TENSORLAB 
path(path,['C:\GitLab\Tools\TensorLab20\TensorLab20']);
path(path,["C:\GitLab\Tools\TensorLab20\TensorLab20"+...
            "\dev\shared_functions"]);
%% define the basis
b  = {'e1'; 'e2'};
ee = cartesianbasis2d(b{1}, b{2});
e1 = ee(1);
e2 = ee(2);
I   = identity(2, ee);

%% MATERIAL

k = 10; % [N/m]


x = [ 0*e1 + 0*e2
      4*e1 + 3*e2     ];


%% SPRING CONNECTION
% We must define a connextivity matrix that explicitly tells which nodes
% are connected with a spring of constant k.
%
% Connecting node i and node j
%
%         string element (physical)              
%
%                     k
%      i  ._________/\/\/\/\__________. j  
%
%      conns = [i j]
%

conns = [1 2];


%% ASSEMBLE SYSTEM MATRIX ON A LOOP



    
    % unit vector in i-j direction in the undeformed configuration
    l0ij = (x(2)-x(1))/norm(x(2)-x(1));

    % compute elementary matrices                    
    K = k*l0ij*l0ij*  [   1    -1 
                         -1     1 ];
    full(tm2sm(K,ee,2))

% NOTE that the following matrix multiplied by disps.
%  provides the internal forces ( = - external force)
%        K = k*l0ij*l0ij*  [  -1     1 
%                              1    -1 ];

   



%% BOUNDARY CONDITIONS


% declare
u = zeros(1, ee, 2, 1); 

% prescribe node 1 to zero (no action required since is declared as 0)
% u(2)=0*e1 + 0*e2 


%% EXTERNAL FORCE
% free node is the node 2

% external forces on (free) node
f2    = dot((10*e1),l0ij*l0ij );
sf2   = full(tm2sm(f2, ee,1))   % [N]

sK22 = full(tm2sm(K(2,2), ee,2))

% tensor matrices 
u(2) = K(2,2)\f2

% scalar matrices
su2   = sK22\sf2

figure(1)
clf
hold on;
plot(dot(x,e1), dot(x,e2));
plot(dot(x+u,e1), dot(x+u,e2), "LineWidth",1.5);
legend('reference','current');
hold off;
grid on;


